@@ -4,7 +4,7 @@ use warnings;
 require Class::Accessor::Fast;
 use base qw/Class::Accessor::Fast/;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 __PACKAGE__->mk_accessors(qw/config/);
 
@@ -15,6 +15,7 @@ use FindBin;
 use UNIVERSAL::require;
 use Carp;
 use Log::Dispatch;
+use Scalar::Util qw/blessed/;
 my $TERM_ANSICOLOR_ENABLED = eval { use Term::ANSIColor; 1; };
 
 sub new {
@@ -173,6 +174,17 @@ sub run_hook {
     }
 }
 
+sub run_hook_and_get_response {
+    my ($self, $hook, @args) = @_;
+
+    $self->log(debug => "Run hook and get response: $hook");
+    for my $action (@{$self->{hooks}->{$hook}}) {
+        my $response = $action->($self, @args);
+        return $response if blessed $response && $response->isa('HTTP::Response');
+    }
+    return; # not finished yet
+}
+
 sub get_hooks {
     my ($self, $hook) = @_;
 
